@@ -1,6 +1,6 @@
 /*********************************************************************
 This file is part of the argtable2 library.
-Copyright (C) 1998,1999,2000,2001,2003,2004,2005 Stewart Heitmann
+Copyright (C) 1998,1999,2000,2001,2003,2004 Stewart Heitmann
 sheitmann@users.sourceforge.net
 
 The argtable2 library is free software; you can redistribute it and/or
@@ -360,9 +360,6 @@ static
 void arg_parse_untagged(int argc, char **argv, struct arg_hdr **table, struct arg_end *endtable)
     {
     int tabindex=0;
-    int errorlast=0;
-    const char *optarglast = NULL;
-    void *parentlast = NULL;
 
     /*printf("arg_parse_untagged(%d,%p,%p,%p)\n",argc,argv,table,endtable);*/
     while (!(table[tabindex]->flag&ARG_TERMINATOR))
@@ -403,31 +400,16 @@ void arg_parse_untagged(int argc, char **argv, struct arg_hdr **table, struct ar
             /* success, move onto next argv[optind] but stay with same table[tabindex] */
             /*printf("arg_parse_untagged(): argtable[%d] successfully matched\n",tabindex);*/
             optind++;
-
-            /* clear the last tentative error */
-            errorlast = 0;
             }
         else
             {
             /* failure, try same argv[optind] with next table[tabindex] entry */
             /*printf("arg_parse_untagged(): argtable[%d] failed match\n",tabindex);*/
             tabindex++;
-
-            /* remember this as a tentative error we may wish to reinstate later */
-            errorlast = errorcode;
-            optarglast = argv[optind];
-            parentlast = parent;
             }
 
         }
 
-    /* if a tenative error still remains at this point then register it as a proper error */
-    if (errorlast)
-        {
-        arg_register_error(endtable,parentlast,errorlast,optarglast);
-        optind++;
-        }
-
     /* only get here when not all argv[] entries were consumed */
     /* register an error for each unused argv[] entry */
     while (optind<argc)
@@ -476,7 +458,6 @@ int arg_parse(int argc, char **argv, void **argtable)
     struct arg_hdr **table = (struct arg_hdr **)argtable;
     struct arg_end *endtable;
     int endindex;
-    char **argvcopy = NULL;
 
     /*printf("arg_parse(%d,%p,%p)\n",argc,argv,argtable);*/
 
@@ -487,38 +468,14 @@ int arg_parse(int argc, char **argv, void **argtable)
     endindex = arg_endindex(table);
     endtable = (struct arg_end*)table[endindex];
 
-    /* alloc space for a local copy of argv[] */
-    argvcopy = malloc(sizeof(char *) * argc);
-    if (argvcopy)
-        {
-        int i;
+    /* parse the command line for tagged optuions */
+    arg_parse_tagged(argc,argv,table,endtable);
 
-        /*
-        Fill in the local copy of argv[]. We need a local copy
-        because getopt rearranges argv[] which adversely affects
-        susbsequent parsing attempts.
-        */
-        for (i=0; i<argc; i++)
-           argvcopy[i] = argv[i];
+    /* parse the command line for untagged options */
+    arg_parse_untagged(argc,argv,table,endtable);
 
-        /* parse the command line (local copy) for tagged options */
-        arg_parse_tagged(argc,argvcopy,table,endtable);
-
-        /* parse the command line (local copy) for untagged options */
-        arg_parse_untagged(argc,argvcopy,table,endtable);
-
-        /* if no errors so far then perform post-parse checks otherwise dont bother */
-        if (endtable->count==0)
-            arg_parse_check(table,endtable);
-
-        /* release the local copt of argv[] */
-        free(argvcopy);
-        }
-    else
-        {
-        /* memory alloc failed */
-        arg_register_error(endtable,endtable,ARG_EMALLOC,NULL);
-        }
+    /* perform post-parse checks */
+    arg_parse_check(table,endtable);
 
     return endtable->count;
     }
@@ -688,7 +645,7 @@ void arg_print_option(FILE *fp, const char *shortopts, const char *longopts, con
  * in: -xvfsd, or -xvf[sd], or [-xvsfd]
  */
 static
-void arg_print_gnuswitch(FILE *fp, struct arg_hdr **table)
+void arg_print_gnuswitch(struct arg_hdr **table)
     {
     int tabindex;
     char *format1="-%c";
@@ -711,7 +668,7 @@ void arg_print_gnuswitch(FILE *fp, struct arg_hdr **table)
             continue;
 
         /* print the short option (only the first short option char, ignore multiple choices)*/
-        fprintf(fp,format1,table[tabindex]->shortopts[0]);
+        printf(format1,table[tabindex]->shortopts[0]);
         format1="%c";
         format2="[%c";
         }
@@ -732,12 +689,12 @@ void arg_print_gnuswitch(FILE *fp, struct arg_hdr **table)
             continue;
 
         /* print first short option */
-        fprintf(fp,format2,table[tabindex]->shortopts[0]);
+        printf(format2,table[tabindex]->shortopts[0]);
         format2="%c";
         suffix="]";
         }
 
-    fprintf(fp,suffix);
+    printf(suffix);
     }
 
 
@@ -747,7 +704,7 @@ void arg_print_syntax(FILE *fp, void **argtable, const char *suffix)
     int i,tabindex;
 
     /* print GNU style [OPTION] string */
-    arg_print_gnuswitch(fp, table);
+    arg_print_gnuswitch(table);
 
     /* print remaining options in abbreviated style */
     for(tabindex=0; table[tabindex] && !(table[tabindex]->flag&ARG_TERMINATOR); tabindex++)
@@ -768,7 +725,7 @@ void arg_print_syntax(FILE *fp, void **argtable, const char *suffix)
             {
             /* print mandatory options */
             for (i=0; i<table[tabindex]->mincount; i++)
-                fprintf(fp, " %s",syntax);
+                printf(" %s",syntax);
 
             /* print optional args enclosed in "[..]" */
             switch ( table[tabindex]->maxcount - table[tabindex]->mincount )
@@ -776,20 +733,20 @@ void arg_print_syntax(FILE *fp, void **argtable, const char *suffix)
                 case 0:
                     break;
                 case 1:
-                    fprintf(fp, " [%s]",syntax);
+                    printf(" [%s]",syntax);
                     break;
                 case 2:
-                    fprintf(fp, " [%s] [%s]",syntax,syntax);
+                    printf(" [%s] [%s]",syntax,syntax);
                     break;
                 default:
-                    fprintf(fp, " [%s]...",syntax);
+                    printf(" [%s]...",syntax);
                     break;
                 }
             }
         }
 
     if (suffix)
-        fprintf(fp, "%s",suffix);
+        printf("%s",suffix);
     }
 
 
@@ -811,7 +768,7 @@ void arg_print_syntaxv(FILE *fp, void **argtable, const char *suffix)
 
         /* print mandatory options */
         for (i=0; i<table[tabindex]->mincount; i++)
-            fprintf(fp," %s",syntax);
+            printf(" %s",syntax);
 
         /* print optional args enclosed in "[..]" */
         switch ( table[tabindex]->maxcount - table[tabindex]->mincount )
@@ -819,19 +776,19 @@ void arg_print_syntaxv(FILE *fp, void **argtable, const char *suffix)
             case 0:
                 break;
             case 1:
-                fprintf(fp, " [%s]",syntax);
+                printf(" [%s]",syntax);
                 break;
             case 2:
-                fprintf(fp, " [%s] [%s]",syntax,syntax);
+                printf(" [%s] [%s]",syntax,syntax);
                 break;
             default:
-                fprintf(fp, " [%s]...",syntax);
+                printf(" [%s]...",syntax);
                 break;
             }
         }
 
     if (suffix)
-        fprintf(fp,"%s",suffix);
+        printf("%s",suffix);
     }
 
 
